<?php
/**
 * @file fragment.module
 * Contains page callbacks for fragment module.
 */

/**
 * Entity form builder.
 *
 * Should be used with entity_ui_get_form().
 *
 * @param $fragment
 *  A fragment entity object.
 * @param $op
 *  One of 'add' or 'edit'.
 */
function fragment_form($form, &$form_state, $fragment, $op = 'edit') {

  if ($op == 'add') {
    // Override the page title for 'add' op.
    // TODO: when http://drupal.org/node/1619628 is committed, this can be
    // removed as well as the page callbacks.
    // Mimics what entity_ui_get_action_title() does once patched.
    $entity_info = entity_get_info('fragment');
    list(, , $bundle_name) = entity_extract_ids('fragment', $fragment);
    $title = t('Add @bundle_name @entity_type', array(
      '@bundle_name' => drupal_strtolower($entity_info['bundles'][$bundle_name]['label']),
      '@entity_type' => drupal_strtolower($entity_info['label']),
    ));
    drupal_set_title($title);
  }

  $form['title'] = array(
    '#title' => t('Title'),
    '#type' => 'textfield',
    '#default_value' => isset($fragment->title) ? $fragment->title : '',
  );

  $form['actions'] = array('#type' => 'actions');
  $form['actions']['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save fragment'),
    '#weight' => 40,
  );

  field_attach_form('fragment', $fragment, $form, $form_state);

  return $form;
}

/**
 * Form API submit callback for the fragment edit form.
 */
function fragment_form_submit(&$form, &$form_state) {
  //dsm($form_state);
  $fragment = entity_ui_form_submit_build_entity($form, $form_state);
  //dsm($fragment);

  // Save the entity.
  $fragment->save();
  // Redirect the the admin UI.
  $form_state['redirect'] = 'admin/content/fragments';
}

/**
 * Page callback to show links to add an entity of a specific bundle.
 *
 * Replacement for entity_ui_bundle_add_page() which shows the product type
 * descriptions that match each of our bundles.
 *
 * @TODO: give this a theme function, similar to theme_node_add_list().
 *
 * @TODO: This can be removed when http://drupal.org/node/1619628 is committed.
 */
function fragment_bundle_add_page() {
  // Set the title, as we're a MENU_LOCAL_ACTION and hence just get tab titles.
  drupal_set_title(t('Add a fragment'));

  // Get entity info for our bundles.
  $info = entity_get_info('fragment');

  $output = '<dl class="fragment-type-list">';
  foreach ($info['bundles'] as $bundle_name => $bundle_info) {
    if (user_access("create $bundle_name fragments")) {
      $bundle_url_str = $bundle_name; // str_replace('_', '-', $bundle_name);
      $output .= '<dt>'
        . l(t('Add @label', array('@label' => $bundle_info['label'])), $info['admin ui']['path'] . '/add/' . $bundle_url_str)
        . '</dt>';
    }
  }
  $output .= '</dl>';

  return $output;
}

/**
 * Page callback to add an entity of a specific bundle.
 *
 * @TODO: This can be removed when http://drupal.org/node/1619628 is committed.
 *
 * @param $entity_type
 *   The type of the entity.
 * @param $bundle_url_str
 *   The bundle machine name converted to a URL string, i.e., with the
 *   underscores converted to hyphens.
 */
function fragment_get_bundle_add_form($entity_type, $bundle_url_str) {
  $bundle_name = str_replace('-', '_', $bundle_url_str);
  $info = entity_get_info($entity_type);
  $bundle_key = $info['entity keys']['bundle'];

  // Make a stub entity of the right bundle to pass to entity_ui_get_form().
  $values = array(
    $bundle_key => $bundle_name,
  );
  $entity = entity_create($entity_type, $values);

  return entity_ui_get_form($entity_type, $entity, 'add');
}

/**
 * Generates the fragment type editing form.
 */
function fragment_type_form($form, &$form_state, $fragment_type, $op = 'edit') {
  if ($op == 'clone') {
    $fragment_type->label .= ' (cloned)';
    $fragment_type->type = '';
  }

  $form['label'] = array(
    '#title' => t('Label'),
    '#type' => 'textfield',
    '#default_value' => isset($fragment_type->label) ? $fragment_type->label : '',
    '#description' => t('The human-readable name of this fragment type.'),
    '#required' => TRUE,
    '#size' => 30,
  );
  // Machine-readable type name.
  $form['type'] = array(
    '#type' => 'machine_name',
    '#default_value' => isset($fragment_type->type) ? $fragment_type->type : '',
    '#maxlength' => 32,
    '#disabled' => ($op == 'clone'),
    '#machine_name' => array(
      'exists' => 'fragment_get_types',
      'source' => array('label'),
    ),
    '#description' => t('A unique machine-readable name for this fragment type. It must only contain lowercase letters, numbers, and underscores.'),
  );

  $form['actions'] = array('#type' => 'actions');
  $form['actions']['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save fragment type'),
    '#weight' => 40,
  );

  if ($op != 'add' && $op != 'clone') {
    $form['actions']['delete'] = array(
      '#type' => 'submit',
      '#value' => t('Delete fragment type'),
      '#weight' => 45,
      '#limit_validation_errors' => array(),
      '#submit' => array('fragment_type_form_submit_delete')
    );
  }

  return $form;
}

/**
 * Form API submit callback for the type form.
 */
function fragment_type_form_submit(&$form, &$form_state) {
  $fragment_type = entity_ui_form_submit_build_entity($form, $form_state);
  // Save and go back.
  $fragment_type->save();
  $form_state['redirect'] = 'admin/structure/fragment-types';
}

/**
 * Form API submit callback for the delete button.
 */
function fragment_type_form_submit_delete(&$form, &$form_state) {
  $form_state['redirect'] = 'admin/structure/fragment-types/manage/' . $form_state['fragment_type']->type . '/delete';
}
